from pathlib import Path
from typing import List

import requests
from bs4 import BeautifulSoup

base_url = "https://www.gob.mx"
articles_url = base_url + "/presidencia/es/archivo/articulos?page="
headers = {
    "User-Agent": (
        "Mozilla/5.0 (Macintosh; "
        "Intel Mac OS X 10_10_1) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/39.0.2171.95 Safari/537.36"
    )
}
raw_path = Path("raw")
raw_path.mkdir(exist_ok=True, parents=True)


def query(page_to_query):
    response = requests.get(articles_url + str(page_to_query), headers=headers)
    data = [
        BeautifulSoup(d.strip()[21:-2].replace('\\"', '"').replace(r"\/", "/").replace("\\n", ""), "html5lib")
        for d in response.text.split("\n")
        if d.strip()[4:10] == "prensa"
    ]

    return data


def clean_url(url):
    qmark = url.find("?")
    if qmark != -1:
        true_link = url[:qmark]
    else:
        true_link = url
    return base_url + true_link


def get_anchors(documents):
    anchors = []
    for doc in documents:
        anchors.extend([clean_url(a["href"]) for a in doc.find_all("a")])
    return anchors


def get_new_urls(url_list, page) -> List[str]:
    page = page or 1
    url_list = Path(url_list)
    url_list.parent.mkdir(exist_ok=True, parents=True)

    last_fetched = None
    old_urls = []
    if url_list.exists():
        with open(url_list) as readable:
            for old_url in readable:
                old_urls.append(old_url.strip())
        last_fetched = old_urls[0]
        print(f"Last url found {last_fetched}")
    else:
        print("No previous urls found, starting from scratch")

    print(f"Starting fetching from page {page}")

    new_urls = []
    stop_crawling = False
    while not stop_crawling:
        results = query(page)
        if not results:
            print("No more urls, no need to dataset anymore")
            break

        if page % 10 == 0:
            print(f"Querying page {page}")

        for link in get_anchors(results):
            if link == last_fetched:
                print("Found a previously crawled page, no need to dataset anymore")
                stop_crawling = True
                break
            new_urls.append(link)

        page += 1

    all_urls = new_urls + old_urls
    with open(url_list, "w") as writable:
        for url in all_urls:
            writable.write(url + "\n")

    return new_urls



from pathlib import Path
from typing import Iterator, List, Union

from mananeras.entities import Mananera, Participacion


def lee_mananera(path: Union[Path, str]) -> Mananera:
    path = Path(path)

    with open(path) as readable:
        title = next(readable).strip()
        author = next(readable).strip()
        date = next(readable).strip()

        participaciones: List[Participacion] = []
        speaker: str = None
        dialogs: List[str] = []
        for line in readable:
            if line.strip() == "---" and speaker is not None:
                participaciones.append(Participacion(speaker, dialogs))
                speaker = None
                dialogs = []
            elif speaker is None:
                speaker = line.strip()
            elif speaker is not None:
                dialogs.append(line.strip())
        if speaker is not None:
            participaciones.append(Participacion(speaker, dialogs))

    return Mananera(titulo=title, autor=author, date_string=date, participaciones=participaciones)


def todas(path: Union[Path, str] = "articulos") -> Iterator[Mananera]:
    path = Path(path)
    if not path.is_dir():
        raise ValueError(f"La ruta {str(path)} no es un directorio")

    for file in path.glob("**/*.txt"):
        yield lee_mananera(file)


def lee_todas(path: Union[Path, str] = "articulos") -> List[Mananera]:
    mananeras = [conf for conf in todas(path)]
    return mananeras



import re
from pathlib import Path
from typing import Tuple

from bs4 import BeautifulSoup, Tag

date_format = re.compile(r"(?P<day>[0-9]{2}) de (?P<month>[a-z]+) de (?P<year>[0-9]{4})")


def extract(raw_input, processed_output_path):
    raw_input = Path(raw_input)
    processed_output_path = Path(processed_output_path)
    processed_output_path.mkdir(exist_ok=True, parents=True)
    existing_files = {str(file).partition("--")[2][:-4] for file in processed_output_path.glob("**/*.txt")}
    for html_file in raw_input.glob("*.html"):
        if html_file.stem in existing_files:
            continue

        all_dialogs, author, date, date_info, title = parse_document(html_file)

        output_file = (
            processed_output_path / date_info["year"] / date_info["month"] / f"{date_info['day']}--{html_file.stem}.txt"
        )
        output_file.parent.mkdir(exist_ok=True, parents=True)
        with open(output_file, "w") as writable:
            writable.write(title + "\n")
            writable.write(author + "\n")
            writable.write(date + "\n")
            for speaker, lines in all_dialogs:
                writable.write("---\n")
                writable.write(speaker + "\n")
                for line in lines:
                    writable.write(line + "\n")


def _c(txt: str) -> str:
    """Clean a string removing whitespace"""
    return txt.replace("\xa0", " ").replace("\xc2", " ").strip()


def _parse_ps(ps: Tag) -> Tuple[str, str]:
    raw_speaker, _, raw_dialog = ps.text.partition(":")
    speaker = None
    dialog = _c(ps.text)
    if raw_speaker.isupper():
        speaker = _c(raw_speaker)
        dialog = _c(raw_dialog or "")
    if all((character == "-" for character in dialog)):
        dialog = None
    return speaker, dialog


def parse_document(file: Path):
    with open(file) as file:
        all_dialogs = []
        soup = BeautifulSoup(file.read(), "html5lib")
        [article_content] = soup.find_all("div", {"class": "pull-left"})[1:-1]
        title = soup.find("h1").text.strip()
        author, date = [dd.text.strip() for dd in soup.find("section", {"class": "border-box"}).find_all("dd")]
        all_ps = article_content.find_all("p")
        date_info = date_format.match(date).groupdict()
        current_speaker = None
        dialogs = []
        for ps in all_ps:
            speaker, dialog = _parse_ps(ps)
            if speaker:
                if dialogs and current_speaker:
                    all_dialogs.append((current_speaker, dialogs))
                current_speaker = speaker
                dialogs = [dialog] if dialog else []
            elif dialog:
                dialogs.append(dialog)

        if dialogs and current_speaker:
            all_dialogs.append((current_speaker, dialogs))

        return all_dialogs, author, date, date_info, title



from pathlib import Path
from typing import List
from urllib.request import urlretrieve


def download_articles(url_list: List[str], download_folder):
    download_folder = Path(download_folder)
    download_folder.mkdir(parents=True, exist_ok=True)

    for url in url_list:
        _, _, name = url.rpartition("/")
        urlretrieve(url, download_folder / f"{name}.html")



from dataclasses import dataclass
from datetime import date, datetime
from typing import List

from dateparser import parse


@dataclass
class Participacion:
    hablante: str
    dialogos: List[str]


@dataclass
class Mananera:
    titulo: str
    autor: str
    date_string: str
    participaciones: List[Participacion]

    @property
    def fecha(self) -> date:
        a: datetime = parse(self.date_string, languages=["es"])
        return a.date()



import logging
import shutil

import click
from kaggle import api

from mananeras.dataset.download_articles import download_articles
from mananeras.dataset.download_urls import get_new_urls
from mananeras.dataset.extract_dialogs import extract


def setup_logger():
    logger = logging.getLogger("mananeras")
    logger.setLevel(logging.INFO)
    return logger


@click.command()
def main():
    logger = setup_logger()

    logger.info("downloading urls")
    new_urls = get_new_urls("urls.txt", 1)
    logger.info("downloading articles")
    download_articles(new_urls, "raw")
    logger.info("processing articles")
    extract("raw", "articulos")
    logger.info("compressing articles")
    shutil.make_archive("data/articulos", "zip", "./articulos")
    logger.info("creating new dataset version")
    api.dataset_create_version("data", "Daily dataset update", dir_mode="zip", quiet=False)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mananeras', 'mananeras.dataset']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0',
 'click>=8.0.1,<9.0.0',
 'dateparser>=1.0.0,<2.0.0',
 'html5lib>=1.1,<2.0',
 'kaggle>=1.5.12,<2.0.0',
 'lxml>=4.6.3,<5.0.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'mananeras',
    'version': '0.1.2',
    'description': 'Las conferencias mananeras del presidente de México',
    'long_description': None,
    'author': 'Antonio Feregrino',
    'author_email': 'antonio.feregrino@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
